#include "../src/config.h"
#include "../src/Noritake_VFD_GUU100.h"
#include <avr/pgmspace.h>
#include "font5x8.h"
#include "font8x8.h"
#include "font20x32_digits.h"
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>

static Noritake_VFD_GUU100 vfd;

#define VERT    0
#define BIG     1
#define DETAIL  1

#define BARS    8
#define LEFT    (DETAIL && VERT==0? 16: 0)
#define TOP     (VERT? 16: 0)
#define LEN     48
#define WID     8
#define GUT     0

static void sample(uint8_t *vals, int l) {
    for (uint8_t i = 0; i < BARS; i++) {
        uint8_t v = 20+(20*(sin((5+i)*l*(3.14/180))));
        v = v % LEN;
        *vals++ = v;
    }
}

static void drawBars(const uint8_t *vals) {
    for (uint8_t i = 0; i < BARS; i++) {
        uint8_t v = vals[i];
        #if VERT == 0
            vfd.fillRect(LEFT, TOP + i * WID, v, WID - GUT, true);
            vfd.fillRect(LEFT + v, TOP + i * WID, LEN - v, WID - GUT, false);
        #else
            vfd.fillRect(LEFT + i * WID, TOP, WID - GUT, v, false);
            vfd.fillRect(LEFT + i * WID, TOP+v, WID - GUT, LEN - v, true);
        #endif
    }
}
static void drawAverage(const uint8_t *vals) {
    #if BIG == 1
        unsigned avg = 0;
        for (uint8_t i = 0; i < BARS; i++)
            avg += vals[i];
        avg /= BARS;
        
        vfd.setFont(font20x32_digits, 20, 32, 4);
        #if NORITAKE_VERTICAL == 0
            vfd.setCursor(NORITAKE_WIDTH-24*2, 16);
            vfd.print(avg, 2, '0');
            vfd.setCursor(NORITAKE_WIDTH-8*2, 16+32);
            vfd.setFont(font5x8, 5, 8, 1);
            vfd.print("db");
        #else
            vfd.setCursor(0, 64 + 16);
            vfd.print(avg, 2, '0');
            vfd.setCursor(24*2, 64+16+32- 8);
            vfd.setFont(font5x8, 5, 8, 1);
            vfd.print("db");
        #endif
    #endif
}
static void drawNumbers(const uint8_t *vals) {
    #if DETAIL == 1 && VERT == 0
        for (uint8_t i = 0; i < BARS; i++) {
            vfd.setFont(font8x8, 8, 8, 0);
            vfd.setCursor(0, i * WID);
            vfd.print(vals[i], 2, ' ');
            vfd.setFont(font5x8, 5, 8, 1);
        }
    #endif
}

static void equalizerDemo() {
    for (;;)
    for (int l=0; l<360; l++) {
        uint8_t vals[BARS];
        
        sample(vals, l);
        drawBars(vals);
        
        if (l % 8 != 0)
            continue;
        
        drawNumbers(vals);
        drawAverage(vals);
    }
}
int main() {
    _delay_ms(500); // Wait for power up
    vfd.reset();
    vfd.init();
    vfd.clearScreen();
    equalizerDemo();
}
